function [alphaExt, clExt, cdExt] = extendpolar(varargin)
% EXTENDPOLAR Extends polars to the full range of AOA ([-180, 180] deg).
%   This function implements different empirical methods to extend the polars
%   outside of their usual range.
%
%   EXTENDPOLAR accepts inputs in the form of a Polar structure (generated with
%   XF2MAT) or values for angles of attack and their associated cl and cd.
%
%   If the input is specified as aoa, cl and cd, the three arrays must have the
%   same number of rows (one for each angle of attack). The columns indicate
%   values for different polars (different Mach, different Reynolds,...).
%   For example, if the cl values are given for 2 different Reynols and there
%   are 100 values for the aoa, cl and cd must be [100 x 2].
%
% IMPORTANT:
%   To be extended, the input polar must AT LEAST containt the points where
%   AOA = 0 deg and stall point.
%
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox">Documentation (README)</a>
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues">Report an issue</a>
% -----
%
% Usage:
%   [ALPHAEXT, CLEXT, CDEXT] = EXTENDPOLAR prompts the user to input a Polar
%   structure and extends the angle of attack (ALPHAEXT), lift coefficient
%   (CLEXT) and drag coefficient (CDEXT).
%
%   [ALPHAEXT, CLEXT, CDEXT] = EXTENDPOLAR(POLAR) uses the POLAR strcture given
%   as input instead of prompting the user to select the file.
%
%   [ALPHAEXT, CLEXT, CDEXT] = EXTENDPOLAR(ALPHA, CL, CD) extends directly the
%   values given in ALPHA, CL and CD over the whole range of angles of attack.
%
%   [...] = EXTENDPOLAR(..., 'method', METHOD) specifies the extension method.
%   The available methods are:
%
%     'Viterna'  - (default) Viterna & Corrigan. 1982. “Fixed Pitch Rotor
%                  Performance of Large Horizontal Axis Wind Turbines”
%
%   [...] = EXTENDPOLAR(..., 'limit', LIMIT) specifies the point from where to
%   extend. The available limits are:
%
%     'last'  - (default) last point of the input polar
%     'stall' - stall point, the points after stall are disregarded
%
% Inputs:
%   Polar  : Polar structure (e.g. output from XF2MAT)
%   alpha  : Vector or array with angles of attack [aoas x 1] or [aoas x nbPolar]
%   cl     : Array with lift coefficients [aoas x nbPolar]
%   cd     : Array with drag coefficients [aoas x nbPolar]
%
% Output:
%   alphaExt : Extended range of angles of attack, IN RADIANS
%   clExt    : Extended lift coefficient array
%   cdExt    : Extended drag coefficient array
%
% Example:
%   EXTENDPOLAR
%   EXTENDPOLAR(Polar)
%   EXTENDPOLAR(Polar, 'method', 'Viterna')
%   EXTENDPOLAR(Polar, 'method', 'Viterna', 'limit', 'stall')
%   EXTENDPOLAR(alpha, cl, cd)
%   EXTENDPOLAR(alpha, cl, cd, 'method', 'Viterna', 'limit', 'stall')
%
% See also: XF2MAT, PLOTPOLARS.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------

% Check number of inputs
narginchk(0,8); % Normally 7, but will disregard cm anyway, so 8 is ok

% Import other functions from this package
import af_tools.findstall
import af_tools.utils.*

% Defaults and constants
OPTION_LIST = {'method','limit'};   % No options for this function

% Extract and validate the inputs
[alpha, cl, cd, ~, idxOpts] = parsepolarinputs(OPTION_LIST,varargin{:});
if isempty(cd)
    error('MATLAB:extendpolar:noCD',['To be extended, the polars must specify '...
        'alpha, cl and cd.']);
end
[method, limit] = parseoptioninputs(varargin{idxOpts:end});

% Check if polars have enough data points to be extended
if sign(min(alpha(:,1))) == sign(max(alpha(:,1)))
    error('MATLAB:extendpolar:noZero',['To be extended, a polar must at '...
        'least contain the AOA = 0 deg.']);
end

try
    findstall(alpha, cl, cd);
catch
    error('MATLAB:extendpolar:noStall',['To be extended, a polar must at '...
        'least contain the stall point.']);
end


%------------------------------------
% Extend polars

if alpha(1,1) == -pi || alpha(end,1) == pi
    % If input already ranges from [-pi,pi], just send it back
    alphaExt = alpha;
    clExt = cl;
    cdExt = cd;
else
    % Extend using the selected method
    switch method
        case 'Viterna'
            for i = 1:size(alpha,2)
                [alphaExt(:,i), clExt(:,i), cdExt(:,i)] = viterna(limit, alpha(:,i), cl(:,i), cd(:,i));
            end
    end
end


end


%--------------------------------------------------------------------------
function [method, limit] = parseoptioninputs(varargin)
% PARSEINPUTS Parses the input and checks their validity

import af_tools.utils.*

% Constants and defaults
DEFAULT_METHOD = 'Viterna';
DEFAULT_LIMIT = 'last';
ALLOWED_METHODS = {'Viterna'};
ALLOWED_LIMITS = {'stall', 'last'};

% Option validator
validMethod = @(x) any(validatestring(x, ALLOWED_METHODS, mfilename()));
validLimit = @(x) any(validatestring(x, ALLOWED_LIMITS, mfilename()));

% Parse options
p = inputParser;
addParameter(p,'method',DEFAULT_METHOD, validMethod);
addParameter(p,'limit',DEFAULT_LIMIT, validLimit);
parse(p,varargin{:});
method = p.Results.method;
limit = p.Results.limit;

end


%--------------------------------------------------------------------------
function [alphaExt, clExt, cdExt] = viterna(limit, alpha, cl, cd)
% VITERNA Implementation of the original Viterna model
% Source:
%   Viterna & Corrigan. 1982. “Fixed Pitch Rotor Performance of Large Horizontal
%   Axis Wind Turbines”.

% Note:
%   The typical subscript 's' used in most definitions is here replaced by the
%   generic 'l' (for limit) as the function can be used either with the stall
%   point or the last available data.
%
% Process (the number will indicate the index of alpha, cl, cd):
%   1. Extend Cl/Cd to ]limit, 90]
%   2. Extend to ]90, 180-limit]
%   3. Extend to ]180-limit,180] (lift must be corrected by linear interpolation)
%   4. Extend to [-limit, -alpha(1)[
%   5. Extend to [-90,-limit[
%   6. Flip 3 and 4 to get the reminaing part
%   EXTRA. Trim negative drag values to the minimum drag of the original polar


% Import other functions from this package
import af_tools.findstall
import af_tools.utils.*

% Constants
DEFAULT_AR = 10; % AR as a rather small impact. 10 is a good compromise
SCALING_CL = 0.7; % Suggested in AeroDyn's manual

% Trim the polars to nonNaN values only
% (some Cl, Cd may be NaN if they come from xf2mat with trimAoas = false)
id = find(~isnan(cl) == 1);
alpha = alpha(id);
cl = cl(id);
cd = cd(id);

% Get limit point
if strcmp(limit, 'stall')
    [alpha_l, cl_l, cd_l] = findstall(alpha, cl, cd);
    % Trim polar values after stall point
    ids = find(alpha>alpha_l);
    alpha(ids) = [];
    cl(ids) = [];
    cd(ids) = [];
elseif strcmp(limit, 'last')
    alpha_l = alpha(end);
    cl_l = cl(end,:);
    cd_l = cd(end,:);
end

% Viterna's coefficients
sas = sin(alpha_l);
cas = cos(alpha_l);

cd_max = 1.11 + 0.018 * DEFAULT_AR;
VitCoeffs.a1 = cd_max/2;
VitCoeffs.b1 = cd_max;
VitCoeffs.a2 = (cl_l - cd_max .* sas .*cas) .* sas ./ cas.^2;
VitCoeffs.b2 = (cd_l - cd_max .* sas.^2) ./ cas;

% Minimum step between 2 AOA
alpha_step = (min(diff(alpha)));

% 1. Extend Cl/Cd to ]limit, 90]
alpha1 = (alpha_l+alpha_step : alpha_step : pi/2)';
[cl1, cd1] = viternaeq(VitCoeffs, alpha1, 1);

% 2. Extend to ]90, 180-limit]
alpha2 = (pi/2+alpha_step : alpha_step : pi-alpha_l)';
[cl2, cd2] = viternaeq(VitCoeffs, pi-alpha2, -SCALING_CL);

% 3. Extend to ]180-limit,180]
alpha3 = (pi-alpha_l+alpha_step : alpha_step : pi)';
[~, cd3] = viternaeq(VitCoeffs, pi-alpha3, 1);
% Correct lift with linear variation
cl3 = (alpha3 - pi) / alpha_l * cl_l * SCALING_CL ;

% 4. Extend to [-limit, -alpha(1)[ (by linear interpolation)
if alpha(1) <= -alpha_l
    alpha4 = [];
    cl4 = [];
    cd4 = [];
    alpha_lneg = alpha(1);
else
    alpha4 = (-alpha_l : alpha_step : alpha(1) - alpha_step)';
    cl4 = interp1([-alpha_l,alpha(1)], [-cl_l*SCALING_CL,cl(1)],alpha4);
    cd4 = interp1([-alpha_l,alpha(1)], [cd_l,cd(1)],alpha4);
    alpha_lneg = -alpha_l;
end

% 5. Extend to [-90,-limit[
alpha5 = (-pi/2 : alpha_step : alpha_lneg-alpha_step)';
[cl5, cd5] = viternaeq(VitCoeffs, alpha5, SCALING_CL);

% 6. Remaining part is simply a relfexion
alpha6 = (-pi : alpha_step : -pi/2-alpha_step)';
cl6 = -flipud([cl2;cl3]);
cd6 = flipud([cd2;cd3]);

% Concatenate everything to get the full extended polars
alphaExt = [alpha6; alpha5; alpha4; alpha; alpha1; alpha2; alpha3];
clExt = [cl6; cl5; cl4; cl; cl1; cl2; cl3];
cdExt = [cd6; cd5; cd4; cd; cd1; cd2; cd3];

% Trim negatives cd to cdmin
cdmin = min(cd);
cdExt(cdExt <= cdmin) = cdmin;

end


%--------------------------------------------------------------------------
function [cl, cd] = viternaeq(VitCoeffs, alpha, clScaling)
% VITERNAEQ Viterna's equation for the extension of the polars
% Source:
%   Viterna & Corrigan. 1982. “Fixed Pitch Rotor Performance of Large Horizontal
%   Axis Wind Turbines”.

cl = VitCoeffs.a1 * sin(2*alpha) + VitCoeffs.a2 * cos(alpha).^2./sin(alpha);
cl = cl * clScaling;

cd = VitCoeffs.b1 * sin(alpha).^2 + VitCoeffs.b2  * cos(alpha);

end
