function [alphaRange, clRange, clSlope] = findcllinearrange(varargin)
% FINDCLLINEARRANGE Finds the linear range of the lift coefficient and returns the range and lift slope.
%   This function approximates the lift coefficient linear range by looking at
%   its first and second derivatives (using hard-coded tolerance values).
%
%   FINDCLLINEARRANGE accepts inputs in the form of a Polar structure (generated
%   with XF2MAT) or values for angles of attack and their associated cl.
%
%   If the input is specified as aoa and cl, the two arrays must have the
%   same number of rows (one for each angle of attack). The columns indicate
%   values for different polars (different Mach, different Reynolds,...).
%   For example, if the cl values are given for 2 different Reynols and there
%   are 100 values for the aoa, cl must be [100 x 2].
%
%   If extended polars are used as input (see EXTENDPOLAR), the function will
%   only look for the zero-lift angle in the range of [-30, 30] deg.
%
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox">Documentation (README)</a>
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues">Report an issue</a>
% -----
%
% Usage:
%   [ALPHA_RANGE, CL_RANGE, CL_SLOPE] = FINDCLLINEARRANGE prompts the user to
%   input a Polar structure and determines the range of angles of attach
%   (ALPHA_RANGE) in which the cl is linear. It also outputs the values of the
%   lift coefficient in that range (CL_RANGE) and the slope of the linear cl
%   (CL_SLOPE).
%
%   [ALPHA_RANGE, CL_RANGE, CL_SLOPE] = FINDCLLINEARRANGE(POLAR) uses the POLAR
%   strcture given as input instead of prompting the user to select the file.
%
%   [ALPHA_RANGE, CL_RANGE, CL_SLOPE] = FINDCLLINEARRANGE(ALPHA, CL) uses
%   directly the values given in ALPHA and CL to determine the linear range
%   characteristics.
%
% Inputs:
%   Polar  : Polar structure (e.g. output from XF2MAT)
%   alpha  : Vector or array with angles of attack [aoas x 1] or [aoas x nbPolar]
%   cl     : Array with lift coefficients [aoas x nbPolar]
%
% Output:
%   alphaRange : Range of angles of attack where cl is considered linear, IN RADIANS
%   clRange    : Range of lift coefficients where cl is considered linear
%   clSlope    : Lift curve slope in the linear range
%
% Example:
%   FINDCLLINEARRANGE
%   FINDCLLINEARRANGE(Polar)
%   FINDCLLINEARRANGE(alpha, cl)
%
% See also: XF2MAT, FINDSTALL, FINDZEROLIFT.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------

narginchk(0,2);

% Import other functions from this package
import af_tools.utils.*

% Defaults and constants
ORIGINAL_RANGE = deg2rad([-30;30]);  % Look for values in this range only
OPTION_LIST = {};   % No options for this function

% Extract and validate the inputs
[alpha, cl] = parsepolarinputs(OPTION_LIST,varargin{:});

% Trim extended polars to original range
% (avoid issues if extended polars are used as input)
[alpha, cl] = trimtorange(ORIGINAL_RANGE, alpha, cl);


%------------------------------------
% Find stall for each polar
alphaRange = zeros(2, size(cl,2));
clRange = zeros(2, size(cl,2));
clSlope = zeros(1, size(cl,2));

for i = 1:size(cl,2)
    [alphaRange(:,i), clRange(:,i), clSlope(i)] = findlinearrangevect(alpha(:,i), cl(:,i));
end

end


% ------------------------------------------------------------------------------
function [alphaRange, clRange, clSlope] = findlinearrangevect(alpha, cl)
% FINDLINEARRANGEVECT Finds the linearrange returns the corresponding range and slope
%   For vectors!

IDEAL_LIFT_SLOPE = 2 * pi;  % Ideal Cl curve, [rad]
TOL_DCL = 0.6;   % Tolerance on the slope (dCl should be at least  > TOL_DCL * IDEAL_LIFT_SLOPE in order to be counted in the linear zone)
TOL_DDCL = 0.01;  % Tolerance on the curvature (ddClTol > ddCl > -ddClTol to be considered linear)
PLOT_POLAR_IF_ERROR = true; % Plot the polar if error so user can see why it fails

% First and second derivatives
dalpha = diff(alpha);
dcl = diff(cl)./dalpha(1);
ddcl = diff(dcl)./dalpha(1);

% Adapt tolerance to angle of attack
ddClTol = TOL_DDCL/dalpha(1)^2;

% Initialize loop
maxFound = false;
minFound = false;

% Initialize linear range start and end (mostly useful for debugging)
alphaRange = [1, length(alpha)];
for i = 1:(length(alpha))-2
    
    % Check if current point is a valid point of the linear range
    isMin = isbound(dcl(i), ddcl(i), TOL_DCL, ddClTol, IDEAL_LIFT_SLOPE);
    isMax = isbound(dcl(end+1-i), ddcl(end+1-i), TOL_DCL, ddClTol, IDEAL_LIFT_SLOPE);
    
    if isMin && ~minFound
        minFound = true;
        alphaRange(1) = i;
    end
    
    if isMax && ~maxFound
        maxFound = true;
        alphaRange(end) = length(dcl) + 2 - i;
    end
    
    % Stop when both min and max were found
    if minFound && maxFound
        break;
    end
end

% Check issues in case linear range was not found
if ~minFound || ~maxFound
    if PLOT_POLAR_IF_ERROR % Plot the polar if error so user can see why it fails
        % Display issue
        figure('Name','linearrange:Debug')
        hold on
        plot(alpha,cl,'--');
        plot(alpha(alphaRange(1):alphaRange(2)),cl(alphaRange(1):alphaRange(2)), 'color', 'r', 'lineWidth', 1.2);
        if ~minFound
            plot(alpha(alphaRange(1)),cl(alphaRange(1)),'s','MarkerFaceColor','b','Markersize',12)
        end
        if ~maxFound
            plot(alpha(alphaRange(2)),cl(alphaRange(2)),'s','MarkerFaceColor','b','Markersize',12)
        end
        grid on
        legend('Polar','Attempt at linear range','Missing point(s)')
    end
    error('MATLAB:findcllinearrange:noRangeFound', ['findcllinearrange was  '...
        'not able to determine the linear range of the given polar. \n' ...
        'Maybe the polar does not have enough points, or it is malformed.\n '...
        'See the attached plot for more details']);
    
end

clRange = cl(alphaRange);
alphaRange = alpha(alphaRange);
clSlope = diff(clRange) / diff(alphaRange);

end


% ------------------------------------------------------------------------------
function bool = isbound(dcl, ddcl, dClTol, ddClTol, idealCl)
% ISBOUND Returns true if the point is a valid bound for the linear range
isSlopeOk = isslopeabovecla(dcl, dClTol, idealCl);
isCurvOk = iscurvaturezero(ddcl, ddClTol);

bool = isSlopeOk && isCurvOk;
end


% ------------------------------------------------------------------------------
function bool = isslopeabovecla(dCl, tol, idealCl)
% ISSLOPEABOVECLA Returns true if the slope is larger than the ideal slope
bool = dCl > tol * idealCl;
end


% ------------------------------------------------------------------------------
function bool = iscurvaturezero(ddCl, tol)
% ISCURVATUREZERO Returns true if the curvature is almost zero
bool = (ddCl < tol) && (ddCl > - tol);
end
