function [alpha_s, cl_s, cd_s] = findstall(varargin)
% FINDSTALL Finds the stall point and returns the corresponding AOA, CL and CD.
%   This function uses Matlab's findpeak in order to determine the position of
%   the stall point.
%
%   FINDSTALL accepts inputs in the form of a Polar structure (generated with
%   XF2MAT) or values for angles of attack and their associated cl and cd. While
%   alpha and cl are required for this function, cd is optional. If it is
%   provided, the function will return the drag value at stall as well.
%
%   If the input is specified as aoa, cl and cd, the three arrays must have the
%   same number of rows (one for each angle of attack). The columns indicate
%   values for different polars (different Mach, different Reynolds,...).
%   For example, if the cl values are given for 2 different Reynols and there
%   are 100 values for the aoa, cl and cd must be [100 x 2].
%
%   If extended polars are used as input (see EXTENDPOLAR), the function will
%   only look for the zero-lift angle in the range of [0, 30] deg.
%
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox">Documentation (README)</a>
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues">Report an issue</a>
% -----
%
% Usage:
%   [ALPHA_S, CL_S, CD_S] = FINDSTALL prompts the user to input a Polar
%   structure and determines the stall angle of attack (ALPHA_S), the lift
%   coefficient value at stall (CL_S) and the drag coefficient at stall (CD_S).
%
%   [ALPHA_S, CL_S, CD_S] = FINDSTALL(POLAR) uses the POLAR strcture given
%   as input instead of prompting the user to select the file.
%
%   [ALPHA_S, CL_S, CD_S] = FINDSTALL(ALPHA, CL, CD) uses directly the values
%   given in in ALPHA, CL and CD to determine the stall.
%
% Inputs:
%   Polar  : Polar structure (e.g. output from XF2MAT)
%   alpha  : Vector or array with angles of attack [aoas x 1] or [aoas x nbPolar]
%   cl     : Array with lift coefficients [aoas x nbPolar]
%   cd     : Array with drag coefficients [aoas x nbPolar]
%
% Output:
%   alpha_s : Stall angle of attack, IN RADIANS
%   cl_s    : Lift at stall angle of attack
%   cd_s    : Drag at stall angle of attack
%
% Example:
%   FINDSTALL
%   FINDSTALL(Polar)
%   FINDSTALL(alpha, cl)
%   FINDSTALL(alpha, cl, cd)
%
% See also: XF2MAT, FINDZEROLIFT, FINDCLLINEARRANGE.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------

narginchk(0,3);

% Import other functions from this package
import af_tools.utils.*

% Defaults and constants
ORIGINAL_RANGE = deg2rad([0;30]);  % Look for values in this range only
OPTION_LIST = {};   % No options for this function

% Extract and validate the inputs
[alpha, cl, cd] = parsepolarinputs(OPTION_LIST,varargin{:});

% Trim extended polars to original range
% (avoid issues if extended polars are used as input)
[alpha, cl, cd] = trimtorange(ORIGINAL_RANGE, alpha, cl, cd);


%------------------------------------
% Find stall for each polar

alpha_s = zeros(1, size(cl,2));
cl_s = zeros(1, size(cl,2));
cd_s = zeros(1, size(cl,2));

if nargin == 2
    for i = 1:size(cl,2)
        [alpha_s(i), cl_s(i)] = findstallvect(alpha, cl(:,i));
    end
    
else
    for i = 1:size(cl,2)
        [alpha_s(i), cl_s(i), cd_s(i)] = findstallvect(alpha(:,i), cl(:,i), cd);
    end
end


end


%--------------------------------------------------------------------------
function [alpha_s, cl_s, cd_s] = findstallvect(alpha, cl, cd)
% FINDSTALLVECT Finds the stall point and returns the corresponding AOA, CL and CD
%   For vectors!

PLOT_POLAR_IF_ERROR = true; % Plot the polar if error so user can see why it fails

% Find stall for each polar using findpeaks
maxFound = false;
minWidth = 0;

% Iterate in order to find only one peak (largest one should indicate the stall)
while ~maxFound && minWidth <= length(alpha)
    [~,locs] = findpeaks(cl,'MinPeakWidth',minWidth);
    if numel(locs)==1
        maxFound = true;
    else
        minWidth = minWidth + 1;
    end
end

if ~maxFound
    % If findpeaks does not work
    if PLOT_POLAR_IF_ERROR
        figure('Name', 'Debug: Polar with no stall')
        plot(rad2deg(alpha), cl)
        title('Polar with no clear stall peak')
        xlabel('AOA [deg]')
        ylabel('C_l [-]')
        grid on
    end
    error('MATLAB:findstall:noStallFound', ['findpeaks did not found any ' ...
        'stall point the polar. Please provide a polar that goes a bit after '...
        'the stall.\n' ...
        'See the attached plot for more details']);
end

alpha_s = alpha(locs);
cl_s = cl(locs);
if nargin == 3
    cd_s = cd(locs);
else
    cd_s = NaN;
end

end
