function [x, y] = nacaairfoil(varargin)
% NACAAIRFOIL Generates the full coordinates of a NACA 4 or 5 digits airfoil.
%   By default, the airfoil is generated with 100 points distributed
%   following a half-cosine rule in order to be finer at the leading and
%   trailing edges. The default output has a zero thickness trailing edge.
%   Coefficients for a finite trailing edge are also implemented.
%   See below for details on how to override the defaults.
%
% Note:
%   This function will always output an odd number of points, even if the user
%   asks for an even value of nPoints. This is because the upper and
%   lower surfaces will have the same number of points, and the leading
%   edge is counted twice (as the first and last element to close the shape)
%   while the trailing edge is counted once.
%
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox">Documentation (README)</a>
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues">Report an issue</a>
% -----
%
% Usage:
%   [X, Y] = NACAAIRFOIL prompts the user for the airfoil denomination, then
%   determines its coordinates.
%
%   [X, Y] = NACAAIRFOIL(DIGITS) determines the coordinates of the airfoil using
%   the airfoil denomination given by DIGITS.
%
%   [X, Y] = NACAAIRFOIL(DIGITS, NPOINTS) determines the NPOINTS coordinates of
%   the airfoil named after DIGITS.
%
%   [...] = NACAAIRFOIL(..., 'spacing', SP) uses the defined spacing rule to
%   determine to position of the coordinates. The available spacings are:
%
%     'linear'     - linear
%     'halfcosine' - (default) finer at leading and trailing edges
%     'cosine'     - finer at leading edge, coarser at the trailing edge
%
%   [...] = NACAAIRFOIL(..., 'zerote', TE) picks between the zero thickness
%   trailing edge or a finite one. The values for zerote are:
%
%     true   - (default) zero thickness trailing edge
%     false  - finite trailing edge
%
% Inputs:
%   digits  : Character vector representing the NACA airfoil (ex: '0012', '24120', '24012')
%   nPoints : Number of output coordinates
%
% Output:
%   [X, Y] : Airfoil coordinates, with 0 <= X <= 1.
%
% Example:
%   NACAAIRFOIL
%   NACAAIRFOIL('0012')
%   NACAAIRFOIL('24012', 120)
%   NACAAIRFOIL('24012', 120, 'spacing', 'halfcosine')
%   NACAAIRFOIL('24012', 120, 'zerote', true)
%
% See also: NACACAMBER.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------

narginchk(0,6);

% Import other functions from this package
import af_tools.nacacamber;
import af_tools.utils.parsenacainputs;

OPTION_LIST = {'spacing', 'zerote'};

% Extract and validate the inputs
[digits, nPoints, idxOpts] = parsenacainputs(OPTION_LIST, varargin{:});
[spacing, zerote] = parseoptioninputs(varargin{idxOpts:end});

%------------------------------------
% Coordinates calculation

% Get camberline (we use ceil((nPoints+1)/2) to ensure the airfoil has the same
% number of points on the upper and lower surfaces.
[xc, yc, gradY] = nacacamber(digits, ceil((nPoints+1)/2), 'spacing', spacing);

% Thickness value (checks were done in nacacamber)
t = str2double(digits(end-1:end)) /100;

% Thickness coefficients
a0 = 0.2969;
a1 = -0.126;
a2 = -0.3516;
a3 = 0.2843;
if zerote
    a4 = -0.1015;
else
    a4 = -0.1036;
end

% Calculate thickness distribution
yt = t/0.2 * (a0*xc.^0.5 + a1*xc + a2*xc.^2 + a3*xc.^3 + a4*xc.^4);

% Upper and lower surface coordinates
theta = atan(gradY);
xu = xc - yt.*sin(theta);
yu = yc + yt.*cos(theta);
xl = xc + yt.*sin(theta);
yl = yc - yt.*cos(theta);

% Full coordinates
x = [xu, fliplr(xl(1:end-1))];
y = [yu, fliplr(yl(1:end-1))];

end


% ------------------------------------------------------------------------------
function [spacing, zerote] = parseoptioninputs(varargin)
% PARSEINPUTS Parses the input and checks their validity

import af_tools.utils.*

% Constants and defaults
DEFAULT_SPACING = 'halfcosine';
DEFAULT_ZEROTE = true;
ALLOWED_SPACING = {'linear', 'cosine', 'halfcosine'};

% Option validator
validLogical = @(x) validateattributes(x, {'logical'},{'scalar'}, mfilename());
validSpacing = @(x) any(validatestring(x, ALLOWED_SPACING, mfilename()));

% Parse options
p = inputParser;
addParameter(p,'spacing',DEFAULT_SPACING, validSpacing);
addParameter(p,'zerote',DEFAULT_ZEROTE, validLogical);
parse(p,varargin{:});
spacing = p.Results.spacing;
zerote = p.Results.zerote;



end
