function [xc, yc, gradY] = nacacamber(varargin)
% NACACAMBER Generates the coordinates of the camberline for a NACA 4 or 5 digits airfoil.
%   By default, the camberline is generated with 100 points distributed
%   following a half-cosine rule in order to be finer at the leading and
%   trailing edges.
%   See below for details on how to override the defaults.
%
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox">Documentation (README)</a>
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues">Report an issue</a>
% -----
%
% Usage:
%   [X, Y] = NACACAMBER prompts the user for the airfoil digits, then calculates
%   the coordinates of the camberline using 100 points spaced with the
%   half-cosine rule.
%
%   [X, Y] = NACACAMBER(DIGITS) calculates the coordinates of the camberline for
%   the NACA airfoil specified by DIGITS, without further input.
%
%   [X, Y] = NACACAMBER(DIGITS, NPOINTS) calculates the coordinates of the
%   the NACA airfoil specified by DIGITS, using NPOINTS points spaced with the
%   half-cosine rule.
%
%   [...] = NACACAMBER(..., 'spacing', SP) uses the defined spacing rule to
%   determine to position of the coordinates. The available spacings are:
%
%     'linear'     - linear
%     'halfcosine' - (default) finer at leading and trailing edges
%     'cosine'     - finer at leading edge, coarser at the trailing edge
%
% Inputs:
%   digits  : Character vector representing the NACA airfoil (ex: '0012', '24120', '24012')
%   nPoints : Number of output coordinates
%
% Output:
%   xc : Camberline X coordinates (0 <= X <= 1)
%   yc : Camberline Y coordinates
%   gradY : Gradient of Y (required to get the whole airfoil coordinates)
%
% Example:
%   NACACAMBER
%   NACACAMBER('0012')
%   NACACAMBER('24012', 120)
%   NACACAMBER('24012', 120, 'spacing', 'halfcosine')
%
% See also: NACAAIRFOIL.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------

narginchk(0,4);

% Import other functions from this package
import af_tools.utils.parsenacainputs;

OPTION_LIST = {'spacing'};

% Extract and validate the inputs
[digits, nPoints, idxOpts] = parsenacainputs(OPTION_LIST, varargin{:});
[spacing] = parseoptioninputs(varargin{idxOpts:end});


%------------------------------------
% Coordinates calculation

if length(digits) == 4
    [xc, yc, gradY] = camber4digits(digits, nPoints, spacing);
elseif length(digits) == 5
    [xc, yc, gradY] = camber5digits(digits, nPoints, spacing);
end

end


% ------------------------------------------------------------------------------
function [spacing] = parseoptioninputs(varargin)
% PARSEINPUTS Parses the input and checks their validity

import af_tools.utils.*

% Constants and defaults
DEFAULT_SPACING = 'halfcosine';
ALLOWED_SPACING = {'linear', 'cosine', 'halfcosine'};

% Option validator
validSpacing = @(x) any(validatestring(x, ALLOWED_SPACING, mfilename()));

% Parse options
p = inputParser;
addParameter(p,'spacing',DEFAULT_SPACING, validSpacing);
parse(p,varargin{:});
spacing = p.Results.spacing;

end


% ------------------------------------------------------------------------------
function [xc, yc, gradY] = camber4digits(digits, nPoints, spacing)
% CAMBER4DIGITS Give the camberline coordinates for a 4 digits airfoil.
%
% NACA Definition:  NACA MPXX
%   M  = Maximum camber divided by 100
%   P  = Position of the max camber divided by 10
%   XX = Maximum thickness divided by 100

% Airfoil parameters
m = str2double(digits(1)) /100;
p = str2double(digits(2)) /10;

% Camberline
xc = getxc(spacing, nPoints);
yc = zeros(size(xc));
gradY = zeros(size(xc));

if m ~= 0
    idx = find(xc <= p); % Front part
    yc(idx) = m/p^2 * (2*p*xc(idx) - xc(idx).^2);
    gradY(idx) = 2*m/p^2 * (p-xc(idx));
    
    idx = find(xc > p); % Aft part
    yc(idx) = m/(1-p)^2 * (1 - 2*p + 2*p*xc(idx)-xc(idx).^2);
    gradY(idx) = 2*m/(1-p)^2 * (p-xc(idx));
end

end


% ------------------------------------------------------------------------------
function [xc, yc, gradY] = camber5digits(digits, nPoints, spacing)
% CAMBER5DIGITS Give the camberline coordinates for a 5 digits airfoil.
%
% NACA Definition:  NACA LPQXX
%   L = Design lift coefficient multiplied by 3/20
%   P = Position of the max camber divided by 20 (ex 3 is 0.15*chord)
%   Q = Camber line (0=standard, 1=reflex)
%   XX = Maximum thickness divided by 100

% Airfoil parameters
cld = str2double(digits(1)) *3/20;
p = str2double(digits(2)) /20;
q = str2double(digits(3));

% Camberline
xc = getxc(spacing, nPoints);
yc = zeros(size(xc));
gradY = zeros(size(xc));

if q == 0 % Standard camber formula
    if p ~= 0
        [r, k1, ~] = getcoeffs5digits(q, p);
        idx = find(xc <= r); % Front part
        yc(idx) = k1/6 * (xc(idx).^3 - 3*r*xc(idx).^2 + r^2*(3-r)*xc(idx));
        gradY(idx) = k1/6 * (3*xc(idx).^2 - 6*r*xc(idx) + r^2*(3-r));
        
        idx=find(xc > r); % Aft part
        yc(idx) = k1*r^3/6 * (1-xc(idx));
        gradY(idx) = -k1*r^3/6;
    end
elseif q == 1 % Reflex camber formula
    if p ~= 0
        [r, k1, k2k1] = getcoeffs5digits(q, p);
        idx = find(xc <= r); % Front part
        yc(idx) = k1/6 * ((xc(idx)-r).^3 - k2k1*(1-r)^3*xc(idx) - r^3*xc(idx) + r^3);
        
        idx=find(xc > r); % Aft part
        yc(idx) = k1/6 * (k2k1*(xc(idx)-r).^3 - k2k1*(1-r)^3*xc(idx) - r^3*xc(idx) + r^3);
    end
end

% Scale yc according to cld
yc = (cld/0.3) * yc;

end


% ------------------------------------------------------------------------------
function xc = getxc(spacing, nPoints)
% GETXC Returns xc vector based on chosen spacing and number of points

switch spacing
    case 'linear'
        xc = linspace(0, 1, (nPoints));
    case 'halfcosine'
        beta = linspace(0, pi, (nPoints));
        xc = (1-cos(beta))/2;
    case 'cosine'
        beta = linspace(0, pi/2, (nPoints));
        xc = 1-cos(beta);
end

end


% ------------------------------------------------------------------------------
function [r,k1,k2k1] = getcoeffs5digits(q, p)
% GETCOEFFS5DIGITS Outputs the value of the various coefficients used in the NACA 5digits formula.
% Source: http://www.airfoiltools.com/airfoil/naca5digit

INTERP_METHOD = 'spline';

if q == 0 % Stanard camber
    coeffs = [0.05, 0.0580, 361.400;
        0.10, 0.1260, 51.640;
        0.15, 0.2025, 15.957;
        0.20, 0.2900, 6.643;
        0.25, 0.3910, 3.230;];
    k2k1 = NaN;
    
elseif q == 1 % Reflex camber
    coeffs = [0.10, 0.1300, 51.990, 0.000764;
        0.15, 0.2170, 15.793, 0.00677;
        0.20, 0.3180, 6.650, 0.0303;
        0.25, 0.4410, 3.191, 0.1355;];
    k2k1 = interp1(coeffs(:,1), coeffs(:,end), p, INTERP_METHOD, 'extrap');
end

r = interp1(coeffs(:,1), coeffs(:,2), p, INTERP_METHOD, 'extrap');
k1 = interp1(coeffs(:,1), coeffs(:,3), p, INTERP_METHOD, 'extrap');

end
