function Polar = xf2mat(varargin)
% XF2MAT Aggregates multiple XFOIL or XFLR5 polar results into a single structure.
%   This function aggregates different airfoil polars obtained with XFOIL or
%   XFLR5 (saved as textfiles) into a single structure for easier handling in
%   other scripts.
%
%   This function can be run without arguments, in this case it will prompt the
%   user for the input.
%   Otherwile, the function asks for two main arguments:
%       1. inputDir: the directory where to look for the files
%       2. inputFiles: the files to load
%
%   The function also comes with options to save automatically the resulting
%   Polar structure and to trim all results to the same range of angles of
%   attack.
%
% Note:
%   If multiple textfiles are selected as input, by default, the function will
%   extend every polar to the maximum range of angles of attack found. Polars
%   with fewer data will be assigned NaN values at the missing angles of attack.
%   If the 'trimAoas' is set to true, the function will find the largest common
%   range of angles of attack accross the various input polars and trim
%   everything to that range. This prevents NaN values in the polars, but
%   removes some data from the most complete inputs.
%
% <a href="https://gitlab.uliege.be/thlamb/airfoil_toolbox">Documentation (README)</a>
% <a href="https://gitlab.uliege.be/thlamb/airfoil_toolbox/-/issues">Report an issue</a>
% -----
%
% Usage:
%   POLAR = XF2MAT prompts the user for all inputs, then aggregate and convert
%   the polar in a single POLAR structure.
%
%   POLAR = XF2MAT(INPUTDIR) aggregates all polars found in INPUTDIR into a
%   single Polar structure.
%
%   POLAR = XF2MAT(INPUTDIR, INPUTFILES) aggregates only the polars specified by
%   INPUTFILES found in INPUTDIR into a single Polar structure.
%
%   POLAR = XF2MAT(..., 'autosave', true) saves the resulting POLAR structure
%   automatically in a MAT-file.
%
%   POLAR = XF2MAT(..., 'trimAoas', true) trims all input polars to the largest
%   common range of angles of attack to prevent NaN values in the ouput.
%
% Inputs:
%   inputDir   : Path of the directory with the XFOIL/XFLR5 data
%   inputFiles : Files to select (ex: '*' (default), '*0012*', {'file1','file2'}, etc)
%
% Output:
%   Polar : Structure collecting the results
%           - Polar.alpha    : Angles of attack [Rad]
%           - Polar.airfoil  : Airfoil name
%           - Polar.reynolds : Reynolds number [-]
%           - Polar.mach     : Mach number [-]
%           - Polar.nCrit    : BL trnansition factor [-]
%           - Polar.cl       : Lift coefficient [-]
%           - Polar.cd       : Drag coefficient [-]
%           - Polar.cm       : Moment coefficient [-]
%           - Polar.origin   : XFOIL / XFLR5
%
% Example:
%   XF2MAT)
%   XF2MAT('test_data')
%   XF2MAT('test_data', '*0012*')
%   XF2MAT('test_data', {'0012_re1e5', '0012_re1e6'})
%   XF2MAT('test_data', 'autosave', true)
%   XF2MAT('test_data', '*', 'trimAoas', true)
%
% See also: EXTENDPOLAR, PLOTPOLARS.

% -----
% XFOIL: http://web.mit.edu/drela/Public/web/xfoil/
% XFLR5: https://www.xflr5.tech/xflr5.htm
% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------

narginchk(0,6);

% Import other functions from this package
import af_tools.utils.*

% Constants and defaults
AUTONAME = true;             % Name saved file automatically
PREVENT_OVERWRITING = true;  % Prevents overwriting of the files during autosave

% Parse inputs
[allFileNames, fullpath, autosave, trimAoas] = parseinputs(varargin{:});

% Convert filenames to string array
allFileNames = string(allFileNames);

% -------------------------------------
% Process XFoil/XFLR5 results

% Number of files found
nbFiles=length(allFileNames);

% Initialize structures
Polar = struct('airfoil', [], ...
    'reynolds',zeros(1,nbFiles), ...
    'mach', zeros(1,nbFiles), ...
    'nCrit', [], ...
    'alpha', [], ...
    'cl', [], ...
    'cd', [], ...
    'cm', []);

Tmp(nbFiles).resultArray=[];

% Initialize AOA range
if trimAoas
    aoaRange = [-180,180];
else
    aoaRange = [-3,3];
end

for i=1:nbFiles
    
    % Load file and extract parameters and results
    fileID = fopen(fullfile(fullpath,allFileNames{i}));
    
    % Scan file header to determine if it is from XFOIL or XFLR5
    % (this also position the pointer at the correct location for reading the rest)
    progName = [];
    while isempty(progName)
        tmpProg = textscan(fileID,'%s', 1, 'Delimiter','\n');
        tmpProgname = char(tmpProg{1});
        if ~isempty(tmpProgname) && strcmp(tmpProgname(1:5),'XFOIL')
            progName = 'XFOIL';
        elseif ~isempty(tmpProgname) && strcmp(tmpProgname(1:5),'xflr5')
            progName = 'XFLR5';
        end
    end
    Polar.origin{1,i} = progName;
    
    % Parse the rest of the results
    tmpAirfoil = textscan(fileID,'%s %s', 1, 'Delimiter','\n:','HeaderLines',1);
    tmpParam = textscan(fileID, '%s %s %f %s %s %f %s %f %s %s %f %*[^\n]', 1, 'HeaderLines', 4); % ! HeaderLines is a cursor. This will read line 9
    Tmp(i).resultArray = cell2mat(textscan(fileID,'%f %f %f %f %f %*[^\n]', 'HeaderLines',4)); % ! HeaderLines is a cursor. This will start reading at line 13
    fclose(fileID); clear fileID;
    
    % Extract the airfoil name
    airfoil_name = char(tmpAirfoil{2});
    Polar.airfoil{1,i} = sanitizestring(airfoil_name);
    
    % Get Mach, Reynolds and Ncrit numbers corresponding to the processed file
    Polar.mach(1,i) = tmpParam{3};
    Polar.reynolds(1,i) = tmpParam{6}*10^tmpParam{8};
    Polar.nCrit(1,i) = tmpParam{11};
    
    % Get the larger common AOA range and the smallest step
    if trimAoas
        aoaRange = findcommonrange(aoaRange, Tmp(i).resultArray);
    else
        aoaRange = findmaxrange(aoaRange, Tmp(i).resultArray);
    end
    
end

% Interpolate results over proper range of AOA if needed
for i=1:nbFiles
    Polar.alpha(:,i) = deg2rad(aoaRange');
    Polar.cl(:,i) = interp1(Tmp(i).resultArray(:,1), Tmp(i).resultArray(:,2), aoaRange);
    Polar.cd(:,i) = interp1(Tmp(i).resultArray(:,1), Tmp(i).resultArray(:,3), aoaRange);
    Polar.cm(:,i) = interp1(Tmp(i).resultArray(:,1), Tmp(i).resultArray(:,5), aoaRange);
    
end
clear Tmp


% Sort everything by increasing Reynolds
[Polar.reynolds, indSort] = sort(Polar.reynolds);
Polar.origin(:) = Polar.origin(indSort);
Polar.airfoil(:) = Polar.airfoil(indSort);
Polar.mach(:)  = Polar.mach(indSort);
Polar.nCrit(:) = Polar.nCrit(indSort);
Polar.cl(:,:)  = Polar.cl(:,indSort);
Polar.cd(:,:)  = Polar.cd(:,indSort);
Polar.cm(:,:)  = Polar.cm(:,indSort);


% -------------------------------------
% Save results automatically

if nargin == 0 % Prompt only if user input is to be expected
    prompt = 'Save the resulting polar? Y/N [Y]: ';
    str = input(prompt,'s');
else
    str = 'N';
end

if ~strcmpi(str,'N') || autosave
    
    % Prompt user for filename
    if ~AUTONAME
        prompt = 'Filename for saved polar?: ';
        filename = input(prompt,'s');
    end
    
    % Automatic filename if user did not input one
    if AUTONAME || isempty(filename)
        order = floor(log10(Polar.reynolds));
        nb = round(Polar.reynolds./10.^order);
        
        if nbFiles == 1
            reStr = sprintf('-Re_%de%d',nb,order);
        else
            reStr = sprintf('-Re_%de%d-%de%d',nb(1),order(1),nb(end),order(end));
        end
        filename_base = [Polar.airfoil{1},reStr];
        
        % Prevent overwriting files
        filename = filename_base;
        if PREVENT_OVERWRITING
            i=0;
            while isfile(appendextension(filename,'.mat'))
                i=i+1;
                filename = [filename_base,'_v',num2str(i)];
            end
        end
        
    end
    filename = appendextension(filename,'.mat');
    save(filename,'Polar')
end


end


% ------------------------------------------------------------------------------
function [allFileNames, fullpath, autosave, trimAoas] = parseinputs(varargin)
% PARSEINPUTS Parse inputs and return the list of files and options

import af_tools.utils.*

% Constants and defaults
DEFAULT_INPUTFILES = '*';    % If not specified, all files of the dir are included by default
DEFAULT_AUTOSAVE = false;    % Do not autosave by default
DEFAULT_TRIMAOAS = false;    % Do not trim AOAs by default
OPTION_LIST = {'autosave','trimAoas'};


% Parse main inputs
hasNoInput = isempty(varargin) || isoption(OPTION_LIST, varargin{1});

if hasNoInput
    % Prompt user for the files to read
    [allFileNames, fullpath] = uigetfile('*.txt', ['Select all XFOIL files to aggregate'], 'MultiSelect', 'on','textFiles');
    idxOpts = 1;
else
    inputDir = varargin{1};
    idxOpts = 2;
    if nargin >= 2 && ~isoption(OPTION_LIST, varargin{2})
        idxOpts = 3;
        inputFiles = varargin{2};
    else
        inputFiles = DEFAULT_INPUTFILES;
    end
    
    % Validate inputDir
    validateattributes(inputDir, {'char','string'},{'nonempty','vector'}, mfilename(), 'inputDir')
    
    
    if ~exist(inputDir, 'dir')
        error('MATLAB:xf2mat:dirNotFound', ['The directory specified as inputDir (%s) can not be found!\n'], inputDir);
    end
    
    % Validate inputFiles
    validateattributes(inputFiles, {'char','cell'},{'nonempty','vector'}, mfilename(), 'inputFiles', 4)
    
    if iscell(inputFiles) &&  ~iscellstr(inputFiles)
        error('MATLAB:xf2mat:wrongInputFiles', ['If inpuFiles is given as a cell array, it must '...
            'contain only character vectors!\n']);
    end
    
    % ------------------------------------------------------
    % Look for files
    
    % Get absolute path
    if contains(inputDir, pwd)
        fullpath = inputDir;
    else
        fullpath = fullfile(pwd, inputDir);
    end
    
    % Convert inputFiles to string for simpler handling
    inputFiles = string(inputFiles);
    
    % Get all files
    AllFiles = [];
    
    for i = 1:length(inputFiles)
        inputFiles(i) = appendextension(inputFiles(i),'.txt'); % Add extension
        dummy = dir(fullfile(fullpath,inputFiles(i)));
        if isempty(dummy)
            warning('MATLAB:xf2mat:FileNotFound', 'Could not find file %s.', fullfile(fullpath,inputFiles(i)));
        end
        AllFiles = [AllFiles; dummy];
    end
    
    if isempty(AllFiles)
        error('MATLAB:xf2mat:noFilesFound', 'Impossible to find ANY file matching the input arguments.');
    end
    
    allFileNames = cell(1,length(AllFiles));
    for i = 1:length(AllFiles)
        allFileNames{i} = AllFiles(i).name;
    end
    
end

% Option validator
validLogical = @(x) validateattributes(x, {'logical'},{'scalar'}, mfilename());

% Parse options
p = inputParser;
addParameter(p,'autosave', DEFAULT_AUTOSAVE, validLogical);
addParameter(p,'trimAoas', DEFAULT_TRIMAOAS, validLogical);
parse(p,varargin{idxOpts:end});
autosave = p.Results.autosave;
trimAoas = p.Results.trimAoas;

end


% ------------------------------------------------------------------------------
function bool = isoption(optList, var)
% ISOPTION Returns true if var is contained in option list

bool = any(strcmpi(var,optList));
end


% ------------------------------------------------------------------------------
function aoaRange = findcommonrange(aoaRange, resArray)
% FINDCOMMONRANGE Find the best common range accors all polars

aoa_step = min(diff(aoaRange));

if resArray(1,1) > aoaRange(1)
    aoaRange(1) = resArray(1,1);
end
if resArray(end,1) < aoaRange(end)
    aoaRange(end) = resArray(end,1);
end
if min(diff(resArray(:,1))) < aoa_step
    aoa_step = min(diff(resArray(:,1)));
end

aoaRange = aoaRange(1):aoa_step:aoaRange(end);

end


% ------------------------------------------------------------------------------
function aoaRange = findmaxrange(aoaRange, resArray)
% FINDMAXRANGE Find the max range between all polars

aoa_step = min(diff(aoaRange));

if resArray(1,1) < aoaRange(1)
    aoaRange(1) = resArray(1,1);
end
if resArray(end,1) > aoaRange(end)
    aoaRange(end) = resArray(end,1);
end
if min(diff(resArray(:,1))) < aoa_step
    aoa_step = min(diff(resArray(:,1)));
end

aoaRange = aoaRange(1):aoa_step:aoaRange(end);

end
