% TEST_FINDSTALL Unitary tests for the findstall function
% Note:
%   Matlab package functionality is not very test-framework friendly.
%   Either the whole package has to be imported in EVERY SINGLE TEST or the
%   functions must be called as <package>.function() everytime.
%   The second option is preferred for the tests as it has the smaller scope.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------


%% Main test function

function tests = test_findstall

tests = functiontests(localfunctions);

end

%% Setup and teardown

function setupOnce(testCase)

addpath('../.'); % Add repository to Matlab Path
addpath('./test_utils'); % Add utils to Matlab Path

% Set random number generator settings.
testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)

rmpath('../.'); % Remove repository from Matlab Path
rmpath('./test_utils'); % Remove utils from Matlab Path

% Restore the random number generator settings.
s = testCase.TestData.currentRNG;
rng(s)

end

function teardown(~)

close all; % Close all figures that would have been openend

end


%% Arguments and errors tests

function test_nargin(testCase)
% Error if too many arguments

dummy = (1:10)';

verifyError(testCase, @() af_tools.findstall(dummy, dummy, dummy, dummy), 'MATLAB:narginchk:tooManyInputs')

end

function test_invalidInputsType(testCase)
% Error if base inputs are of the wrong type

empty = [];
scal = 0;
logi = true;
char = 'test';
Struct = struct;

% 1. Polar (struct) or alpha (numeric)
verifyError(testCase, @() af_tools.findstall(empty), 'MATLAB:parsepolarinputs:expectedNonempty')
verifyError(testCase, @() af_tools.findstall(logi), 'MATLAB:parsepolarinputs:invalidType')
verifyError(testCase, @() af_tools.findstall(char), 'MATLAB:parsepolarinputs:invalidType')

% 2. cl (numeric)
verifyError(testCase, @() af_tools.findstall(scal, logi), 'MATLAB:parsepolarinputs:invalidType')
verifyError(testCase, @() af_tools.findstall(scal, char), 'MATLAB:parsepolarinputs:invalidType')
verifyError(testCase, @() af_tools.findstall(scal, Struct), 'MATLAB:parsepolarinputs:invalidType')

% 3. cd (numeric)
verifyError(testCase, @() af_tools.findstall(scal, scal, logi), 'MATLAB:parsepolarinputs:invalidType')
verifyError(testCase, @() af_tools.findstall(scal, scal, char), 'MATLAB:parsepolarinputs:invalidType')
verifyError(testCase, @() af_tools.findstall(scal, scal, Struct), 'MATLAB:parsepolarinputs:invalidType')

end


function test_invalidInputSize(testCase)
% Error if cl and cd do not have the same number of rows than alpha

dummy = (1:20)';
wrong = rand(8,4);

verifyError(testCase, @() af_tools.findstall(dummy, wrong, dummy), 'MATLAB:parsepolarinputs:incorrectNumrows')
verifyError(testCase, @() af_tools.findstall(dummy, dummy, wrong), 'MATLAB:parsepolarinputs:incorrectNumrows')

end


function test_invalidAlphaInput(testCase)
% Error if alpha is not increasing (row directions)

dummy = -(1:20)';

verifyError(testCase, @() af_tools.findstall(dummy), 'MATLAB:parsepolarinputs:expectedIncreasing')

end


function test_invalidPolarInput(testCase)
% Error if missing fields in the polar

Dummy = dummysinglepolar();
DummyAlpha = rmfield(Dummy,'alpha');
DummyCl = rmfield(Dummy,'cl');
DummyCd = rmfield(Dummy,'cd');

verifyError(testCase, @() af_tools.findstall(DummyAlpha), 'MATLAB:nonExistentField')
verifyError(testCase, @() af_tools.findstall(DummyCl), 'MATLAB:nonExistentField')
verifyError(testCase, @() af_tools.findstall(DummyCd), 'MATLAB:nonExistentField')

end


% ------------------------------------------------------------------------------
function test_noStall(testCase)
% Error if impossible to find the stall

Polar = dummysinglepolar();
Polar.cl = ones(size(Polar.cl));

verifyError(testCase, @() af_tools.findstall(Polar), 'MATLAB:findstall:noStallFound')

end


%% Test if function returns expected outputs

function test_correctResults(testCase)
% Check if function works as intended on basic cases

alpha = deg2rad(0:120)/4;
cl = sin(4*alpha);
cd = cos(4*alpha);

[alpha_s, cl_s, cd_s] = af_tools.findstall(alpha, cl, cd);

verifyEqual(testCase, alpha_s, (pi/2)/4, 'AbsTol', 1e-9)
verifyEqual(testCase, cl_s, 1, 'AbsTol', 1e-9)
verifyEqual(testCase, cd_s, 0, 'AbsTol', 1e-9)

end
