% TEST_PARSEPOLARINPUTS Unitary tests for the parsepolarinputs sub-function
%   Due to the importance of this small utility function, many tests were
%   written. This should ensure maximum complicance in all possible context.
% Note:
%   Matlab package functionality is not very test-framework friendly.
%   Either the whole package has to be imported in EVERY SINGLE TEST or the
%   functions must be called as <package>.function() everytime.
%   The second option is preferred for the tests as it has the smaller scope.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------


%% Main test function

function tests = test_parsepolarinputs

tests = functiontests(localfunctions);

end


%% Setup and teardown

function setupOnce(testCase)

addpath('../.'); % Add repository to Matlab Path
addpath('./test_utils'); % Add utils to Matlab Path

% Set random number generator settings
testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)

rmpath('../.'); % Remove repository from Matlab Path
rmpath('./test_utils'); % Remove utils from Matlab Path

% Restore the random number generator settings
s = testCase.TestData.currentRNG;
rng(s)

end

function teardown(~)

close all; % Close all figures that would have been openend

end


%% Check if errors and warnings are raised

function test_invalidOptsList(testCase)
% Error if optList is not a cell array or a string array


empty = [];
scal = 0;
logi = true;

% OptList must be a char, string or cell array
verifyError(testCase, @() af_tools.utils.parsepolarinputs(empty), 'MATLAB:parsepolarinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(scal), 'MATLAB:parsepolarinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(logi), 'MATLAB:parsepolarinputs:invalidType')

end


function test_invalidInputTypeWhenNoOpts(testCase)
% Error if any input is of the wrong type

EMPTY_OPTS = {};
empty = [];
scal = 0;
logi = true;
char = 'test';
Struct = struct;

% 1. Polar (struct) or alpha (numeric)
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, empty), 'MATLAB:parsepolarinputs:expectedNonempty')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, logi), 'MATLAB:parsepolarinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, char), 'MATLAB:parsepolarinputs:invalidType')

% 2. cl (numeric)
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, scal, logi), 'MATLAB:parsepolarinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, scal, char), 'MATLAB:parsepolarinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, scal, Struct), 'MATLAB:parsepolarinputs:invalidType')

% 3. cd (numeric)
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, scal, scal, logi), 'MATLAB:parsepolarinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, scal, scal, char), 'MATLAB:parsepolarinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, scal, scal, Struct), 'MATLAB:parsepolarinputs:invalidType')

% 4. cm (numeric)
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, scal, scal, scal, logi), 'MATLAB:parsepolarinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, scal, scal, scal, char), 'MATLAB:parsepolarinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, scal, scal, scal, Struct), 'MATLAB:parsepolarinputs:invalidType')

end


function test_invalidInputSizeWhenNoOpts(testCase)
% Error if cl, cd and cm do not have the same number of rows than alpha

EMPTY_OPTS = {};

dummy = (1:20)';
wrong = rand(8,4);

verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, dummy, wrong, dummy, dummy), 'MATLAB:parsepolarinputs:incorrectNumrows')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, dummy, dummy, wrong, dummy), 'MATLAB:parsepolarinputs:incorrectNumrows')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, dummy, dummy, dummy, wrong), 'MATLAB:parsepolarinputs:incorrectNumrows')

end


function test_invalidAlphaInputWhenNoOpts(testCase)
% Error if alpha is not increasing (row directions)

EMPTY_OPTS = {};
dummy = -(1:20)';

verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, dummy), 'MATLAB:parsepolarinputs:expectedIncreasing')

end


function test_invalidPolarInputWhenNoOpts(testCase)
% Error if missing fields in the polar

EMPTY_OPTS = {};

Dummy = dummysinglepolar();
DummyAlpha = rmfield(Dummy,'alpha');
DummyCl = rmfield(Dummy,'cl');
DummyCd = rmfield(Dummy,'cd');
DummyCm = rmfield(Dummy,'cm');

verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, DummyAlpha), 'MATLAB:nonExistentField')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, DummyCl), 'MATLAB:nonExistentField')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, DummyCd), 'MATLAB:nonExistentField')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, DummyCm), 'MATLAB:nonExistentField')

end


function test_invalidInputTypeWithOpts(testCase)
% Error if any input is of the wrong type

EMPTY_OPTS = {'option1','option2'};
empty = [];
scal = 0;
logi = true;
char = 'test';
Struct = struct;

% 1. Polar (struct) or alpha (numeric)
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, empty), 'MATLAB:parsepolarinputs:expectedNonempty')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, logi), 'MATLAB:parsepolarinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, char), 'MATLAB:parsepolarinputs:invalidType')

% 2. cl (numeric)
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, scal, logi), 'MATLAB:parsepolarinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, scal, char), 'MATLAB:parsepolarinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, scal, Struct), 'MATLAB:parsepolarinputs:invalidType')

% 3. cd (numeric)
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, scal, scal, logi), 'MATLAB:parsepolarinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, scal, scal, char), 'MATLAB:parsepolarinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, scal, scal, Struct), 'MATLAB:parsepolarinputs:invalidType')

% 4. cm (numeric)
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, scal, scal, scal, logi), 'MATLAB:parsepolarinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, scal, scal, scal, char), 'MATLAB:parsepolarinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, scal, scal, scal, Struct), 'MATLAB:parsepolarinputs:invalidType')

end


function test_invalidInputSizeWithOpts(testCase)
% Error if cl, cd and cm do not have the same number of rows than alpha

EMPTY_OPTS = {'option1','option2'};
dummy = (1:20)';
wrong = rand(8,4);

verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, dummy, wrong, dummy, dummy), 'MATLAB:parsepolarinputs:incorrectNumrows')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, dummy, dummy, wrong, dummy), 'MATLAB:parsepolarinputs:incorrectNumrows')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, dummy, dummy, dummy, wrong), 'MATLAB:parsepolarinputs:incorrectNumrows')

end


function test_invalidAlphaInputWithOpts(testCase)
% Error if alpha is not increasing (row directions)

EMPTY_OPTS = {'option1','option2'};
dummy = -(1:20)';

verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, dummy), 'MATLAB:parsepolarinputs:expectedIncreasing')

end


function test_invalidPolarInputWithOpts(testCase)
% Error if missing fields in the polar

EMPTY_OPTS = {'option1','option2'};

Dummy = dummysinglepolar();
DummyAlpha = rmfield(Dummy,'alpha');
DummyCl = rmfield(Dummy,'cl');
DummyCd = rmfield(Dummy,'cd');
DummyCm = rmfield(Dummy,'cm');

verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, DummyAlpha), 'MATLAB:nonExistentField')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, DummyCl), 'MATLAB:nonExistentField')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, DummyCd), 'MATLAB:nonExistentField')
verifyError(testCase, @() af_tools.utils.parsepolarinputs(EMPTY_OPTS, DummyCm), 'MATLAB:nonExistentField')

end


%% Test if function returns expected outputs

function test_noInput(~)
% TODO: Find how to test if Matlab open uigetfiles prompt

end


function test_emptyInputsInPolar(testCase)
% If empty inputs, return empty vectors

EMPTY_OPTS = {};
AOA = 1:10;
EmptyPolar = struct('alpha',AOA,'cl',[],'cd',[],'cm',[]);

[alpha, cl, cd, cm, idxOpts] = af_tools.utils.parsepolarinputs(EMPTY_OPTS, EmptyPolar);

verifyEqual(testCase, alpha, AOA');
verifyEqual(testCase, cl, []);
verifyEqual(testCase, cd, []);
verifyEqual(testCase, cm, []);
verifyEqual(testCase, idxOpts, 2);

end


function test_emptyInputsArrays(testCase)
% If empty inputs, return empty vectors

EMPTY_OPTS = {};
AOA = 1:10;
cl = [];
cd = [];
cm = [];

[alpha, cl, cd, cm, idxOpts] = af_tools.utils.parsepolarinputs(EMPTY_OPTS, AOA, cl, cd, cm);

verifyEqual(testCase, alpha, AOA');
verifyEqual(testCase, cl, []);
verifyEqual(testCase, cd, []);
verifyEqual(testCase, cm, []);
verifyEqual(testCase, idxOpts, 5);

end


function test_normalInputsWithoutOpts(testCase)
% Return inputs properly if no option is set

TEST_OPTS = {};
alphaIn = 1:10;
clIn = rand(10,2);
cdIn = rand(10,2);
cmIn = rand(10,2);

[alphaOut, clOut, cdOut, cmOut, idxOpts] = af_tools.utils.parsepolarinputs(TEST_OPTS, alphaIn, clIn, cdIn, cmIn);

verifyEqual(testCase, alphaOut, [alphaIn;alphaIn]');
verifyEqual(testCase, clOut, clIn);
verifyEqual(testCase, cdOut, cdIn);
verifyEqual(testCase, cmOut, cmIn);
verifyEqual(testCase, idxOpts, 5);

end


function test_normalInputsWithOptsNotInArgs(testCase)
% Return inputs properly if options are set but no present in the inputs

TEST_OPTS = {'option'};
alphaIn = 1:10;
clIn = rand(10,2);
cdIn = rand(10,2);
cmIn = rand(10,2);

[alphaOut, clOut, cdOut, cmOut, idxOpts] = af_tools.utils.parsepolarinputs(TEST_OPTS, alphaIn, clIn, cdIn, cmIn);

verifyEqual(testCase, alphaOut, [alphaIn;alphaIn]');
verifyEqual(testCase, clOut, clIn);
verifyEqual(testCase, cdOut, cdIn);
verifyEqual(testCase, cmOut, cmIn);
verifyEqual(testCase, idxOpts, 5);

end


function test_normalInputsWithOptsInArgs(testCase)
% Return inputs properly if options are set and present in the inputs

TEST_OPTS = {'option'};
alphaIn = 1:10;
value = 'optionvalue';

[alphaOut, clOut, cdOut, cmOut, idxOpts] = af_tools.utils.parsepolarinputs(TEST_OPTS, alphaIn, 'option', value);

verifyEqual(testCase, alphaOut, alphaIn');
verifyEqual(testCase, clOut, []);
verifyEqual(testCase, cdOut, []);
verifyEqual(testCase, cmOut, []);
verifyEqual(testCase, idxOpts, 2);

end
