% TEST_TRIMTORANGE Unitary tests for the trimtorange sub-function
% Note:
%   Matlab package functionality is not very test-framework friendly.
%   Either the whole package has to be imported in EVERY SINGLE TEST or the
%   functions must be called as <package>.function() everytime.
%   The second option is preferred for the tests as it has the smaller scope.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------


%% Main test function

function tests = test_trimtorange

tests = functiontests(localfunctions);

end


%% Setup and teardown

function setupOnce(testCase)

addpath('../.'); % Add repository to Matlab Path
addpath('./test_utils'); % Add utils to Matlab Path

% Set random number generator settings
testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)

rmpath('../.'); % Remove repository from Matlab Path
rmpath('./test_utils'); % Remove utils from Matlab Path

% Restore the random number generator settings
s = testCase.TestData.currentRNG;
rng(s)

end

function teardown(~)

close all; % Close all figures that would have been openend

end


%% Arguments and error tests

function test_nargout(testCase)
% Ensure that the function throws an error when too many output

range = [-5;5];
dummy = -10:10;

[~, ~] = verifyError(testCase, @() af_tools.utils.trimtorange(range, dummy), 'MATLAB:nargoutchk:tooManyOutputs');

end


%% Test for correct output

function test_rangeInAOA(testCase)
% Trim properly if range is fully contained in alpha

range = [-1,1]';
alpha = (-10:10)';
cl = alpha + 10;
cd = alpha - 10;

expAlphaTrim = (-1:1)';
[alphaTrim, clTrim, cdTrim] = af_tools.utils.trimtorange(range, alpha, cl, cd);

verifyEqual(testCase, alphaTrim, expAlphaTrim);
verifyEqual(testCase, clTrim, expAlphaTrim+10);
verifyEqual(testCase, cdTrim, expAlphaTrim-10);

end


function test_rangeOutAOA(testCase)
% If range is all out of AOA, just return the input

range = [-20,-15]';
alpha = (-10:10)';
cl = alpha + 10;
cd = alpha - 10;

[alphaTrim, clTrim, cdTrim] = af_tools.utils.trimtorange(range, alpha, cl, cd);

verifyEqual(testCase, alphaTrim, alpha);
verifyEqual(testCase, clTrim, cl);
verifyEqual(testCase, cdTrim, cd);


end


function test_rangePartOutAOA(testCase)
% Trim properly if partially out of alpha

range = (-20:0)';
alpha = (-10:10)';
cl = alpha + 10;
cd = alpha - 10;

expAlphaTrim = (-10:0)';
[alphaTrim, clTrim, cdTrim] = af_tools.utils.trimtorange(range, alpha, cl, cd);

verifyEqual(testCase, alphaTrim, expAlphaTrim);
verifyEqual(testCase, clTrim, expAlphaTrim+10);
verifyEqual(testCase, cdTrim, expAlphaTrim-10);

end


function test_rangeEqualAOA(testCase)
% Trim properly if range is equal to alpha

alpha = (-10:10)';
range = (-10:10)';
cl = (20:40)';
cd = (-20:0)';

[alphaTrim, clTrim, cdTrim] = af_tools.utils.trimtorange(range, alpha, cl, cd);

verifyEqual(testCase, alphaTrim, alpha);
verifyEqual(testCase, clTrim, cl);
verifyEqual(testCase, cdTrim, cd);

end


function test_scalarRange(testCase)
% Trim properly if range is a scalar

range = 0;
alpha = (-10:10)';
cl = alpha + 10;
cd = alpha - 10;

expAlphaTrim = 0;
[alphaTrim, clTrim, cdTrim] = af_tools.utils.trimtorange(range, alpha, cl, cd);

verifyEqual(testCase, alphaTrim, expAlphaTrim);
verifyEqual(testCase, clTrim, expAlphaTrim+10);
verifyEqual(testCase, cdTrim, expAlphaTrim-10);

end
